<?php 
/*
Plugin Name: Scriblio III Catalog Importer
Plugin URI: http://about.scriblio.net/
Description: Imports catalog content directly from a III web OPAC, no MaRC export/import needed.
Version: .02
Author: Casey Bisson
Author URI: http://maisonbisson.com/blog/
*/
/* Copyright 2007 Casey Bisson & Plymouth State University

	This program is free software; you can redistribute it and/or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation; either version 2 of the License, or 
	(at your option) any later version. 

	This program is distributed in the hope that it will be useful, 
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
	GNU General Public License for more details. 

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software 
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
*/ 

// The importer 
class ScribIII_import { 
	var $importer_code = 'scribimporter_iii'; 
	var $importer_name = 'Scriblio III Catalog Importer'; 
	var $importer_desc = 'Imports catalog content directly from a III web OPAC, no MaRC export/import needed. <a href="http://about.scriblio.net/wiki/">Documentation here</a>.'; 
	 
	// Function that will handle the wizard-like behaviour 
	function dispatch() { 
		if (empty ($_GET['step'])) 
			$step = 0; 
		else 
			$step = (int) $_GET['step']; 

		// load the header
		$this->header();

		switch ($step) { 
			case 0 :
				$this->greet();
				break;
			case 1 : 
				$this->iii_start(); 
				break;
			case 2:
				$this->iii_getrecords(); 
				break; 
			case 3:
				$this->ktnxbye(); 
				break; 
		} 

		// load the footer
		$this->footer();
	} 

	function header() {
		echo '<div class="wrap">';
		echo '<h2>'.__('Scriblio III Importer').'</h2>';
	}

	function footer() {
		echo '</div>';
	}

	function greet() {
		echo '<div class="narrow">';
		echo '<p>'.__('Howdy! Start here to import records from a Innovative Interfaces (III) ILS system into Scriblio.').'</p>';
		echo '<p>'.__('This has not been tested much. Mileage may vary.').'</p>';

		echo '<form action="admin.php?import='. $this->importer_code .'&amp;step=1" method="post">';
		echo '<p class="submit"><input type="submit" name="next" value="'.__('Next &raquo;').'" /></p>';
		echo '</form>';

		if(!function_exists('mb_convert_encoding')){
			echo '<br /><br />';	
			echo '<p>'.__('This PHP install does not support <a href="http://php.net/manual/en/ref.mbstring.php">multibyte string functions</a>, including <a href="http://php.net/mb_convert_encoding">mb_convert_encoding</a>. Without that function, this importer can&#039;t convert the character encoding from records in the ILS into UTF-8. Accented characters will likely not import correctly.').'</p>';
		}

		echo '<br /><br />';	
		echo '<form action="admin.php?import=scribimporter&amp;step=3" method="post">';
		echo '<p class="submit">or jump immediately to <input type="submit" name="next" value="'.__('Publish Harvested Records &raquo;').'" /> <br />'. __('(goes to default Scriblio importer)').'</p>';
		echo '</form>';
		echo '</div>';
	}

	function ktnxbye() {
		echo '<div class="narrow">';
		echo '<p>'.__('All done.').'</p>';
		echo '</div>';
	}

	function iii_start(){
		$prefs = get_option('scrib_iiiimporter');
		$temp['scrib_iii-sourceprefix'] = $prefs['scrib_iii-sourceprefix'];
		$temp['scrib_iii-sourceinnopac'] = $prefs['scrib_iii-sourceinnopac'];
		$temp['scrib_iii-warnings'] = array();
		$temp['scrib_iii-errors'] = array();
		$temp['scrib_iii-record_start'] = $prefs['scrib_iii-record_start'];
		$temp['scrib_iii-record_end'] = '';
		$temp['scrib_iii-records_harvested'] = 0;
		update_option('scrib_iiiimporter', $temp);

		$this->iii_options();
	}

	function iii_options(){
	
		$prefs = get_option('scrib_iiiimporter');

		echo '<div class="narrow">';

		echo '<p>'.__('All Scriblio records have a &#039;sourceid,&#039; a unique alphanumeric string that&#039;s used to avoid creating duplicate records and, in some installations, link back to the source system for current availability information.').'</p>';
		echo '<p>'.__('The sourceid is made up of two parts: the prefix that you assign, and the bib number from the Innopac. Theoretically, you chould gather records from 1,296 different systems, it&#039;s a big world.').'</p>';

		echo '<form name="myform" id="myform" action="admin.php?import='. $this->importer_code .'&amp;step=2" method="post">';
?>
<p><label for="scrib_iii-sourceinnopac">The Innopac base hostname:<br /><input type="text" name="scrib_iii-sourceinnopac" id="scrib_iii-sourceinnopac" value="<?php echo attribute_escape( $prefs['scrib_iii-sourceinnopac'] ); ?>" /><br />example: lola.plymouth.edu</label></p>
<p><label for="scrib_iii-sourceprefix">The source prefix:<br /><input type="text" name="scrib_iii-sourceprefix" id="scrib_iii-sourceprefix" value="<?php echo attribute_escape( $prefs['scrib_iii-sourceprefix'] ); ?>" /><br />example: bb (must be two characters, a-z and 0-9 accepted)</label></p>
<p><label for="scrib_iii-record_start">Start with bib number:<br /><input type="text" name="scrib_iii-record_start" id="scrib_iii-record_start" value="<?php echo attribute_escape( $prefs['scrib_iii-record_start'] ); ?>" /></label></p>
<p><label for="scrib_iii-record_end">End:<br /><input type="text" name="scrib_iii-record_end" id="scrib_iii-record_end" value="<?php echo attribute_escape( $prefs['scrib_iii-record_end'] ); ?>" /></label></p>
<p><br /><label for="scrib_iii-debug"><input type="checkbox" name="scrib_iii-debug" id="scrib_iii-debug" value="1" /> Turn on debug mode.</label></p>
<?php
		echo '<p class="submit"><input type="submit" name="next" value="'.__('Next &raquo;').'" /></p>';
		echo '</form>';
		echo '</div>';
	}

	function iii_getrecords(){
		if(empty($_POST['scrib_iii-sourceprefix']) || empty($_POST['scrib_iii-sourceinnopac']) || empty($_POST['scrib_iii-record_start'])){
			echo '<p>'.__('Sorry, there has been an error.').'</p>';
			echo '<p><strong>Please complete all fields</strong></p>';
			return;
		}

		// save these settings so we can try them again later
		$prefs = get_option('scrib_iiiimporter');
		$prefs['scrib_iii-sourceprefix'] = strtolower(ereg_replace('[^a-z|A-Z|0-9]', '', $_POST['scrib_iii-sourceprefix']));
		stripslashes($_POST['scrib_iii-sourceprefix']);
		$prefs['scrib_iii-sourceinnopac'] = ereg_replace('[^a-z|A-Z|0-9|-|\.]', '', $_POST['scrib_iii-sourceinnopac']);
		$prefs['scrib_iii-record_start'] = (int) $_POST['scrib_iii-record_start'];
		$prefs['scrib_iii-record_end'] = (int) $_POST['scrib_iii-record_end'];
		update_option('scrib_iiiimporter', $prefs);

		$interval = 50;
		if(!$prefs['scrib_iii-record_end'] || ($prefs['scrib_iii-record_end'] - $prefs['scrib_iii-record_start'] < $interval))
			$interval = $prefs['scrib_iii-record_end'] - $prefs['scrib_iii-record_start'];
		if($prefs['scrib_iii-record_end'] - $prefs['scrib_iii-record_start'] < 1)
			$interval = 1;

		ini_set('memory_limit', '1024M');
		set_time_limit(0);
		ignore_user_abort(TRUE);

		error_reporting(E_ERROR);

		if(!empty($_POST['scrib_iii-debug'])){

			$host =  $prefs['scrib_iii-sourceinnopac'];
			$bibn = (int) $prefs['scrib_iii-record_start'];

			echo '<h3>The III Record:</h3><pre>';			
			echo $this->iii_get_record($host, $bibn);
			echo '</pre><h3>The Tags and Display Record:</h3><pre>';

			$test_pancake = $this->iii_parse_record($this->iii_get_record($host, $bibn), $bibn);
			print_r($test_pancake);
			echo '</pre>';
			
			echo '<h3>The Raw Excerpt:</h3>'. $test_pancake['the_excerpt'] .'<br /><br />';
			echo '<h3>The Raw Content:</h3>'. $test_pancake['the_content'] .'<br /><br />';
			echo '<h3>The SourceID: '. $test_pancake['the_sourceid'] .'</h3>';
			
			// bring back that form
			echo '<h2>'.__('III Options').'</h2>';
			$this->iii_options();
		
		}else{
			// import with status
			$host =  ereg_replace('[^a-z|A-Z|0-9|-|\.]', '', $_POST['scrib_iii-sourceinnopac']);

			$count = 0;
			echo "<p>Reading $interval records from {$prefs['scrib_iii-sourceinnopac']}. Please be patient.<br /><br /></p>";
			echo '<ol>';
			for($bibn = $prefs['scrib_iii-record_start'] ; $bibn <= ($prefs['scrib_iii-record_start'] + $interval) ; $bibn++ ){
				if($record = $this->iii_get_record( $host , $bibn )){
					$bibr = $this->iii_parse_record( $record , $bibn );
					echo "<li>{$bibr['the_title']} {$bibr['the_sourceid']}</li>";
					$count++;
				}
			}
			echo '</ol>';
			
			$prefs['scrib_iii-warnings'] = array_merge($prefs['scrib_iii-warnings'], $this->warn);
			$prefs['scrib_iii-errors'] = array_merge($prefs['scrib_iii-errors'], $this->error);
			$prefs['scrib_iii-records_harvested'] = $prefs['scrib_iii-records_harvested'] + $count;
			update_option('scrib_iiiimporter', $prefs);

			if($bibn < $prefs['scrib_iii-record_end']){
				$prefs['scrib_iii-record_start'] = $prefs['scrib_iii-record_start'] + $interval;
				update_option('scrib_iiiimporter', $prefs);

				$this->iii_options();
				?>
				<div class="narrow"><p><?php _e("If your browser doesn't start loading the next page automatically click this link:"); ?> <a href="javascript:nextpage()"><?php _e("Next Records"); ?></a> </p>
				<script language='javascript'>
				<!--
	
				function nextpage() {
					document.getElementById('myform').submit();
				}
				setTimeout( "nextpage()", 1250 );
	
				//-->
				</script>
				</div>
				<?php
			}else{
				$this->iii_done();
				?>
				<script language='javascript'>
				<!--
					window.location='#complete';
				//-->
				</script>
				</div>
				<?php
			}
		}
	}

	function iii_get_record($host, $bibn){
		if(strpos(file_get_contents('http://'. $host .'/record=b'. $bibn), 'No Such Record')){
			$this->warn = 'Record number '. $bibn .' is suppressed or deleted.';
			return(FALSE);
		}

		$recordurl = 'http://'. $host .'/search/.b'. $bibn .'/.b'. $bibn .'/1%2C1%2C1%2CB/marc~b'. $bibn;

		if(function_exists('mb_convert_encoding'))
			$record = mb_convert_encoding(file_get_contents($recordurl), 'UTF-8');
		else
			$record = file_get_contents($recordurl);

		if(!empty($record)){
			preg_match('/<pre>([^<]*)/', $record, $stuff);
			return($stuff[1]);
		}
		$this->error = 'Host unreachable or no parsable data found for record number '. $bibn .'.';
		return(FALSE);
	}

	function iii_done(){
		$prefs = get_option('scrib_iiiimporter');

		// click next
		echo '<div class="narrow">';

		if(count($prefs['scrib_iii-warnings'])){
			echo '<h3 id="warnings">Warnings</h3>';
			echo '<a href="#complete">bottom</a> &middot; <a href="#errors">errors</a>';
			echo '<ol><li>';
			echo implode($prefs['scrib_iii-warnings'], '</li><li>');
			echo '</li></ol>';
		}

		if(count($prefs['scrib_iii-errors'])){
			echo '<h3 id="errors">Errors</h3>';
			echo '<a href="#complete">bottom</a> &middot; <a href="#warnings">warnings</a>';
			echo '<ol><li>';
			echo implode($prefs['scrib_iii-errors'], '</li><li>');
			echo '</li></ol>';
		}		

		echo '<h3 id="complete">'.__('Processing complete.').'</h3>';
		echo '<p>'. $prefs['scrib_iii-records_harvested'] .' '.__('records harvested.').' with '. count($prefs['scrib_iii-warnings']) .' <a href="#warnings">warnings</a> and '. count($prefs['scrib_iii-errors']) .' <a href="#errors">errors</a>.</p>';

		echo '<p>'.__('Continue to the next step to publish those harvested catalog entries.').'</p>';

		echo '<form action="admin.php?import=scribimporter&amp;step=3" method="post">';
		echo '<p class="submit"><input type="submit" name="next" value="'.__('Publish Harvested Records &raquo;').'" /> <br />'. __('(goes to default Scriblio importer)').'</p>';
		echo '</form>';

		echo '</div>';
	}


	function iii_clean($input){
//$str = preg_replace('/\s\s+/', ' ', $str);
		while(strpos($input, '  '))
			$input = str_replace('  ', ' ', $input);
		return($input);
	}

	function iii_parse_row($lineray){
		$marcrow = array();
		unset($lineray[0]);
		foreach($lineray as $element){
			$count[$element{0}]++;
			$elementname = $element{0}.$count[$element{0}];
			$marcrow[$elementname] = trim($this->iii_clean(substr($element, 1)));
		}
		return($marcrow);
	}

	function iii_parse_record($marcrecord, $bibn){
		global $scrib_import;
	
		$atomic = array();
		
		$marcrecord = str_replace("\n       ", ' ', $marcrecord);
		
		$details = explode("\n", $marcrecord);
		array_pop($details);
		array_shift($details);

		$details[0] = str_replace('LEADER ', '000    ', $details[0]);
		foreach($details as $line){		
			unset($lineray);
			unset($marc);

			$line = trim($line);
			$lineray = trim($this->iii_clean(substr($line, 0, 3) . '|' . substr($line, 4, 2) . '|a' . substr($line, 7)));

			$lineray = explode('|', ereg_replace('\.$', '', $lineray));
			unset($lineray[1]);

			if($lineray[0] > 99)
				$line = trim($this->iii_clean(substr($line, 7)));
			
			// Authors
			if(($lineray[0] == 100) || ($lineray[0] == 110)){
				$marc = $this->iii_parse_row($lineray);
				$temp = ereg_replace(',$', '', $marc['a1'] .' '. $marc['d1']);
				$atomic['author'][] = $temp;
			}else if($lineray[0] == 110){
				$marc = $this->iii_parse_row($lineray);
				$temp = $marc['a1'];
				$atomic['author'][] = $temp;
			}else if(($lineray[0] > 699) && ($lineray[0] < 721)){
				$marc = $this->iii_parse_row($lineray);
				$temp = ereg_replace(',$', '', $marc['a1'] .' '. $marc['d1']);
				$atomic['author'][] = $temp;
			
			//Standard Numbers
			}else if($lineray[0] == 10){
				$marc = $this->iii_parse_row($lineray);
				$atomic['lccn'][] = ereg_replace('[^0-9]', '', $marc['a1']);
			}else if($lineray[0] == 20){
				$marc = $this->iii_parse_row($lineray);
				$temp = trim($marc['a1']) . ' ';
				$temp = ereg_replace('[^0-9|x|X]', '', strtolower(substr($temp, 0, strpos($temp, ' '))));
				$atomic['isbn'][] = $temp;
			}else if($lineray[0] == 22){
				$marc = $this->iii_parse_row($lineray);
				$temp = trim($marc['a1']) . ' ';
				$temp = ereg_replace('[^0-9|x|X|\-]', '', strtolower(substr($temp, 0, strpos($temp, ' '))));
				$atomic['issn'][] = $temp;
				$temp = trim($marc['y1']) . ' ';
				$temp = ereg_replace('[^0-9|x|X|\-]', '', strtolower(substr($temp, 0, strpos($temp, ' '))));
				$atomic['issn'][] = $temp;
				$temp = trim($marc['z1']) . ' ';
				$temp = ereg_replace('[^0-9|x|X|\-]', '', strtolower(substr($temp, 0, strpos($temp, ' '))));
				$atomic['issn'][] = $temp;
				$atomic['issn'] = array_filter( $atomic['issn'] );
			
			//Titles
			}else if($lineray[0] == 245){
				$marc = $this->iii_parse_row($lineray);
				$temp = ucwords(trim(ereg_replace('/$', '', $marc['a1']) .' '. trim(ereg_replace('/$', '', $marc['b1']))));
				$atomic['title'][] = $temp;
				$atomic['attribution'][] = $marc['c1'];
			}else if($lineray[0] == 240){
				$marc = $this->iii_parse_row($lineray);
				$temp = trim(ereg_replace('/$', '', $marc['a1'] .' '. $marc['b1']));
				$atomic['alttitle'][] = $temp;
			}else if($lineray[0] == 246){
				$marc = $this->iii_parse_row($lineray);
				$temp = trim(ereg_replace('/$', '', $marc['a1'] .' '. $marc['b1']));
				$atomic['alttitle'][] = $temp;
			}else if(($lineray[0] > 719) && ($lineray[0] < 741)){
				$marc = $this->iii_parse_row($lineray);
				$temp = $marc['a1'];
				$atomic['alttitle'][] = $marc['a1'];
			
			//Dates
			}else if($lineray[0] == 260){
				$marc = $this->iii_parse_row($lineray);
				if($marc['c1']){
					$temp = str_pad(substr(ereg_replace('[^0-9]', '', $marc['c1']), 0, 4), 4 , '5');
					if( $temp < date('Y') + 2 )
						$atomic['pubyear'][] = $temp;
				}
			}else if($lineray[0] == 5){
				$atomic['acqdate'][] = $line{7}.$line{8}.$line{9}.$line{10} .'-'. $line{11}.$line{12} .'-'. $line{13}.$line{14};
			}else if($lineray[0] == 8){
				$temp = intval(substr($line, 14, 4));
				if($temp)
					$atomic['pubyear'][] = substr($line, 14, 4);
			
			//Subjects
			}else if(($lineray[0] > 599) && ($lineray[0] < 700)){
				$marc = $this->iii_parse_row($lineray);
				$atomic['subject'][] = implode(' -- ', $marc);
				
				if($atomic['subjkey']){
					$atomic['subjkey'] = array_unique(array_merge($atomic['subjkey'], array_values($marc)));
				}else{
					$atomic['subjkey'] = array_values($marc);
				}	
		
			//URLs
			}else if($lineray[0] == 856){
				$marc = $this->iii_parse_row($lineray);
				unset($temp);
				$temp['href'] = $temp['title'] = str_replace(' ', '', $marc['u1']);
				$temp['title'] = trim(parse_url( $temp['href'] , PHP_URL_HOST), 'www.');
				if($marc['31'])
					$temp['title'] = $marc['31'];
				if($marc['z1'])
					$temp['title'] = $marc['z1'];
				$atomic['url'][] = '<a href="'. $temp['href'] .'">'. $temp['title'] .'</a>';
		
			//Notes
			}else if(($lineray[0] > 299) && ($lineray[0] < 400)){
				$marc = $this->iii_parse_row($lineray);
				$atomic['physdesc'][] = implode(' ', array_values($marc));
			}else if(($lineray[0] > 399) && ($lineray[0] < 500)){
				$marc = $this->iii_parse_row($lineray);
				$atomic['title'][] = implode("\n", array_values($marc));
			}else if(($lineray[0] > 799) && ($lineray[0] < 841)){
				$marc = $this->iii_parse_row($lineray);
				$atomic['series'][] = implode("\n", array_values($marc));
			}else if(($lineray[0] > 499) && ($lineray[0] < 600)){
				if($lineray[0] == 504)
					continue;
				if($lineray[0] == 505){
					$atomic['contents'][] = str_replace(array('> ','>  ','>   '), '>', '<li>'. str_replace('--', "</li>\n<li>", trim(str_replace(array('|t', '|r'), '', $line))) .'</li>');
					continue;
				}
				$atomic['notes'][] = $line;
			}
			
			//Format
			if((!$atomic['format']) && ($lineray[0] > 239) && ($lineray[0] < 246)){
				$marc = $this->iii_parse_row($lineray);
				$temp = ucwords(strtolower(str_replace('[', '', str_replace(']', '', $marc['h1']))));
				
				if(eregi('^book', $temp)){
					$format = 'Book';
					$formats = 'Books';

				}else if(eregi('^micr', $temp)){
					$format = 'Microform';

				}else if(eregi('^electr', $temp)){
					$format = 'Website';
					$formats = 'Websites';

				}else if(eregi('^vid', $temp)){
					$format = 'Video';
				}else if(eregi('^motion', $temp)){
					$format = 'Video';

				}else if(eregi('^audi', $temp)){
					$format = 'Audio';
				}else if(eregi('^cass', $temp)){
					$format = 'Audio';
				}else if(eregi('^phono', $temp)){
					$format = 'Audio';
				}else if(eregi('^record', $temp)){
					$format = 'Audio';
				}else if(eregi('^sound', $temp)){
					$format = 'Audio';

				}else if(eregi('^carto', $temp)){
					$format = 'Map';
					$formats = 'Maps';
				}else if(eregi('^map', $temp)){
					$format = 'Map';
					$formats = 'Maps';
				}else if(eregi('^globe', $temp)){
					$format = 'Map';
					$formats = 'Maps';

				}else if($temp){
					$format = 'Classroom Material';
					//$format = $temp;
				}

				if(!$formats)
					$formats = $format;
				
				if($format){
					$atomic['format'][] = $format;
					$atomic['formats'][] = $formats;
				}
			}

			if($lineray[0] == '008' && substr($lineray[2], 22,1) == 'p'){
				$atomic['format'][] = 'Journal';
				$atomic['formats'][] = 'Journals';
			}
		}
		// end the big loop

		$atomic = array_filter($atomic);

		// Records without acqdates are reserves by course/professor
		// we _can_ import them, but they don't have enough info
		// to be findable or display well.
		if(!$atomic['acqdate'][0] && !$atomic['author'][0]){
			$this->warn = 'Record number '. $bibn .' contains no catalog date or author info, skipped.';
			return(FALSE);
		}
		if(count($atomic) < 4){
			$this->warn = 'Record number '. $bibn .' has too little cataloging data, skipped.';
			return(FALSE);
		}

		if(!$atomic['acqdate'][0])
			$atomic['acqdate'][0] = date('Y') -1 .'-'. date('m') .'-'. date('m') ;

		if(!$atomic['format'][0]){
			$atomic['format'][0] = 'Book';
		}
	
		$atomic['pubyear'] = array_unique($atomic['pubyear']);
		if(empty($atomic['pubyear'][0]))
			$atomic['pubyear'][0] = substr($atomic['acqdate'][0],0,4);

		if($atomic['pubyear'][0])
			$atomic['pubdate'][0] = $atomic['pubyear'][0].substr($atomic['acqdate'][0],4);

		if($atomic['alttitle'])
			$atomic['title'] = array_unique(array_merge($atomic['title'], $atomic['alttitle']));
		
		$atomic['the_sourceid'] = substr(ereg_replace('[^a-z|0-9]', '', strtolower($_POST['scrib_iii-sourceprefix'])), 0, 2) . $bibn;

		if(!empty($atomic['title']) && !empty($atomic['the_sourceid'])){

			$atomic['tags']['subj'] = $atomic['subjkey'];
			$atomic['tags']['auth'] = $atomic['author'];
			$atomic['tags']['isbn'] = $atomic['isbn'];
			$atomic['tags']['issn'] = $atomic['issn'];
			$atomic['tags']['title'] = $atomic['title'];
			$atomic['tags']['format'] = $atomic['format'];
			$atomic['tags']['pubyear'] = $atomic['pubyear'];

			if($sweets = $scrib_import->get_sweets($atomic['isbn'])){
				if(!empty($sweets['img']));
					$atomic['img'] = $sweets['img'];
				if(!empty($sweets['summary'])){
					$atomic['shortdescription'] = html_entity_decode($sweets['summary']);
				}
			}
			$atomic['the_title'] = $atomic['title'][0];
			$atomic['the_pubdate'] = $atomic['pubdate'][0];
			$atomic['the_acqdate'] = $atomic['acqdate'][0];
	
			$atomic['the_excerpt'] = $scrib_import->the_excerpt($atomic);
			$atomic['the_content'] = $scrib_import->the_content($atomic);

			$scrib_import->insert_harvest($atomic);
			return($atomic);
		}else{
			$this->error = 'Record number '. $bibn .' couldn&#039;t be parsed.';
			return(FALSE);
		}

	}

	// Default constructor 
	function ScribIII_import() {
		// nothing
	} 
} 

// Instantiate and register the importer 
include_once(ABSPATH . 'wp-admin/includes/import.php'); 
if(function_exists('register_importer')) { 
	$scribiii_import = new ScribIII_import(); 
	register_importer($scribiii_import->importer_code, $scribiii_import->importer_name, $scribiii_import->importer_desc, array (&$scribiii_import, 'dispatch')); 
} 

add_action('activate_'.plugin_basename(__FILE__), 'scribiii_importer_activate'); 

function scribiii_importer_activate() { 
	global $wp_db_version, $scribiii_import; 
	 
	// Deactivate on pre 2.3 blogs 
	if($wp_db_version<6075) { 
		$current = get_settings('active_plugins'); 
		array_splice($current, array_search( plugin_basename(__FILE__), $current), 1 ); 
		update_option('active_plugins', $current); 
		do_action('deactivate_'.plugin_basename(__FILE__));		 
		return(FALSE);
	}
} 

?>
